import { getSystemPath, normalize, virtualFs } from '@angular-devkit/core';
import { TempScopedNodeJsSyncHost } from '@angular-devkit/core/node/testing';
import { HostTree } from '@angular-devkit/schematics';
import { SchematicTestRunner, UnitTestTree } from '@angular-devkit/schematics/testing';
import * as shx from 'shelljs';
import { SchematicsTestNGConfig, SchematicsTestTsConfig } from '../config';

describe('secondary entry points checks', () => {
  let runner: SchematicTestRunner;
  let host: TempScopedNodeJsSyncHost;
  let tree: UnitTestTree;
  let tmpDirPath: string;
  let previousWorkingDir: string;
  let warnOutput: string[];

  beforeEach(() => {
    runner = new SchematicTestRunner('test', require.resolve('../../../migration.json'));
    host = new TempScopedNodeJsSyncHost();
    tree = new UnitTestTree(new HostTree(host));

    writeFile('/tsconfig.json', JSON.stringify(SchematicsTestTsConfig));
    writeFile('/angular.json', JSON.stringify(SchematicsTestNGConfig));

    warnOutput = [];
    runner.logger.subscribe(logEntry => {
      if (logEntry.level === 'warn') {
        warnOutput.push(logEntry.message);
      }
    });

    previousWorkingDir = shx.pwd();
    tmpDirPath = getSystemPath(host.root);

    shx.cd(tmpDirPath);

    writeFakeAngular();
  });

  afterEach(() => {
    shx.cd(previousWorkingDir);
    shx.rm('-r', tmpDirPath);
  });

  function writeFakeAngular(): void { writeFile('/node_modules/@angular/core/index.d.ts', ``); }

  function writeFile(filePath: string, contents: string): void {
    host.sync.write(normalize(filePath), virtualFs.stringToFileBuffer(contents));
  }

  // tslint:disable-next-line:no-any
  async function runMigration(): Promise<any> {
    await runner.runSchematicAsync('migration-v9', {}, tree).toPromise();
  }

  describe('secondary entry points', () => {

    it('warning should be printed', async() => {
      writeFile('/index.ts', `
      import { NzTSType } from 'ng-zorro-antd/core';
      import { isNil } from 'ng-zorro-antd/core';
      import { NzNoAnimationModule } from 'ng-zorro-antd/core/no-animation';
      import { NzTreeNodeOptions, NzTreeNode } from 'ng-zorro-antd/core';
      import { NzTreeNodeOptions, NzTreeNode } from 'ng-zorro-antd/tree';
      import { NzTreeNodeOptions, NzTreeNode } from 'ng-zorro-antd';
     `);

      await runMigration();

      const warnLines = ['index.ts@2:7', 'index.ts@3:7', 'index.ts@5:7'];
      const warnOutputString = warnOutput.toString();
      warnLines.forEach(line => {
        expect(warnOutputString).toContain(`/${line} - The entry-point "ng-zorro-antd/core" is remove`);
      });

      expect(warnOutputString).not.toContain(`/index.ts@6`);
      expect(warnOutputString).toContain(`/index.ts@7:7 - The entry-point "ng-zorro-antd" is remove`);

    });

  });

});
